#!/usr/bin/env python3

import os
import time
import sys
import signal
import configparser

from .config_migration import migrate_config
from .wwan_modem import WwanModem
from .ublox_wwan_modem import UbloxWwanModem
from .platform import SimSelector
from .modem_manager import ModemManager, ModemManagerException
from .logger import logger

CONF_PATH = '/run/wwan/'
SETUP_PATH = '/etc/wwan/'


def write_config_file(file, label, value):
    line = (label + ':').ljust(20) + value + '\n'
    file.write(line)


def dump_firmware(config_file, fw):
    version, status = fw
    if status == WwanModem.OBS_FW:
        config = '{} (Obsolete)'.format(version)
    elif status == WwanModem.DEP_FW:
        config = '{} (Deprecated)'.format(version)
    elif status == WwanModem.UNTESTED_FW:
        config = '{} (Untested)'.format(version)
    else:
        config = version
    write_config_file(config_file, 'Firmware', config)


def dump_config(modem):
    logger.info("Writing modem config to /run")
    os.makedirs(CONF_PATH, exist_ok=True)
    config_file = open(CONF_PATH + modem.get_name() + '.config', 'w')
    vendor = modem.get_vendor()
    write_config_file(config_file, 'Vendor', vendor)
    write_config_file(config_file, 'Model', modem.get_model())
    dump_firmware(config_file, modem.get_firmware())

    if vendor == "u-blox":
        write_config_file(config_file, 'Default APN', modem.get_apn())
        write_config_file(config_file, 'Mode', modem.get_mode())
        write_config_file(config_file, 'USBprofile', modem.get_usb_profile())


def init_modem(name):
    sim_reload = False

    config = configparser.ConfigParser()
    config.read(SETUP_PATH + name + '.conf')

    sim_sel = SimSelector()
    modem = WwanModem(name)

    # SIM selection
    sim0 = config.get('sim', 'SIM') or 'auto'
    if sim0 == 'auto':
        # Switch to eSIM and reset if main is not available
        sim_reload |= sim_sel.auto_connect(modem)
    else:
        sim_reload |= sim_sel.connect(sim0)

    if modem.get_vendor() == 'u-blox':
        logger.info("u-blox modem")
        modem = UbloxWwanModem(modem)
    return modem, sim_reload

def configure_modem(modem):
    rst = False
    config = configparser.ConfigParser()
    config.read(SETUP_PATH + modem.get_name() + '.conf')

    # Vendor specific configuration
    rst |= modem.do_vendor_config(config)

    # Reset if required
    if rst:
        logger.info("reset required")
        modem.reset()

    # Setup APN
    apn = config.get('apn', 'apn') or None
    if apn:
        user = config.get('apn', 'user') or None
        password = config.get('apn', 'password') or None
        modem.set_apn(apn, user, password)

    # Write setup to /run
    dump_config(modem)

    modem.close_serial()


def start_modem_manager(modem):
    while True:
        try:
            ModemManager.monitor_startup()
            break
        except ModemManagerException:
            recover_modem(modem)

def stop(modem):
    ModemManager.stop()
    modem.open_serial()
    modem.power_down()
    modem.close_serial()

def start(modem):
    modem.power_up()
    modem.open_serial()
    configure_modem(modem)
    ModemManager.start()

def recover_modem(modem):
    stop(modem)
    time.sleep(5)
    start(modem)

def daemon_loop(modem):
    monitoring_inhibited = False
    def inhibit_handler(sig, frame):
        """ Request form other tools to access the modem directly (eg. firmware update) """
        nonlocal monitoring_inhibited
        if sig == signal.SIGUSR1:
            logger.info("MM monitoring inhibited")
            monitoring_inhibited = True
            ModemManager.stop()
        else:
            logger.info("MM monitoring restart")
            monitoring_inhibited = False
            ModemManager.start()
    signal.signal(signal.SIGUSR1, inhibit_handler)
    signal.signal(signal.SIGUSR2, inhibit_handler)

    mm_monitor = ModemManager()
    while True:
        time.sleep(10)

        if monitoring_inhibited:
            continue

        try:
            mm_monitor.monitor()
        except ModemManagerException:
            logger.warning("ModemManager cannot recover modem, doing power cycle")
            recover_modem(modem)

def daemonize(modem):
    n = os.fork()
    if n > 0:
        f = open('/run/{}.pid'.format(modem.get_name()), 'w')
        f.write(str(n))
        f.close()
        return

    def signal_handler(sig, frame):
        stop(modem)
        sys.exit(0)

    signal.signal(signal.SIGTERM, signal_handler)

    daemon_loop(modem)


def main():
    if sys.argv[1] and sys.argv[1].startswith('wwan'):
        name = sys.argv[1]
        ModemManager.stop()
        migrate_config(name)
        modem, sim_reload = init_modem(name)
        if sim_reload:
            logger.info("Reloading sim card")
            modem.reload_sim()
        configure_modem(modem)
        start_modem_manager(modem)
        daemonize(modem)


if __name__ == '__main__':
    main()
