#!/usr/bin/env python3
""" Module handling ublox modems """

import re
import time
from .wwan_modem import WwanModem
from .exceptions import UnexpectedFirmwareVersion
from .logger import logger

LATEST_FW = "17.00,A01.01"
RECOMMENDED_FW = "17.00,A01.01"

SIM_SPEED_256K = 2

class UbloxWwanModem(WwanModem):
    """ Ublox modem definition """

    def __init__(self, parent):
        super(UbloxWwanModem, self).__init__(parent)

    def do_vendor_config(self, config):
        """ Vendor specific configuration (usb mode, sim speed, etc) """
        rst = False
        usb_profile = config.get('ublox', 'usb_profile') or 'ECM'
        mode = config.get('ublox', 'mode') or 'Bridge'

        rst |= self.set_usb_profile(usb_profile)
        rst |= self.set_mode(mode)

        fw = self._get_firmware_number()
        if fw >= 17000100:
            rst |= self.set_sim_speed()
        else:
            logger.warning("The firmware is too old to limit the sim speed")

        return rst

    def get_apn(self):
        """ Get default bearer APN """
        output = self.send_command(b'AT+UCGDFLT?')
        x = re.search('","(.*)",', output)
        return x.group(1)

    def set_apn(self, apn, user=None, password=None):
        """ Set default bearer APN """
        apn = apn.encode('utf-8')
        if user is None:
            auth = b'0'
            user = b'NULL'
            password = b'NULL'
        else:
            auth = b'1'
            user = '"{}"'.format(user).encode('utf-8')
            if password is None:
                password = b'NULL'
            else:
                password = '"{}"'.format(password).encode('utf-8')

        # Switching to airplane mode
        self.send_command(b'AT+CFUN=4', no_answer=True)

        cmd = b'AT+UCGDFLT=0,"IP","' + apn + b'",0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,' + auth + b',' + user + b',' + password + b',""'

        self.send_command(cmd, no_answer=True)

        # Switching back to full featured mode
        self.send_command(b'AT+CFUN=1', no_answer=True)
        time.sleep(1)  # Make sure modem is fully responsive

    def reload_sim(self):
        """ Reload sim card when it has been switched """
        self.send_command(b'AT+CFUN=19', no_answer=True)
        time.sleep(2)
        self.send_command(b'AT+CFUN=1', no_answer=True)
        time.sleep(2)

    def reset(self):
        """ Reset the modem """
        try:
            self.send_command(b'AT+CFUN=16', no_answer=True)
        except Exception:
            pass  # When resetting, the modem does not answer

        # Wait until modem reappears
        self.close_serial()
        self.wait_on_down()
        self.wait_on_up()
        self.open_serial()

    def get_sim_speed(self):
        """ Get the configured max speed of the sim card communication. """
        resp = self.send_command(b'AT+UDCONF=86,7')
        regex = re.search('UDCONF:\\s86,7,(\\d)', resp)
        speed = int(regex.group(1))
        return speed

    def set_sim_speed(self):
        """ Sets the max speed for SIM card communication, returns True if reset required """
        current = self.get_sim_speed()
        if current == SIM_SPEED_256K:
            logger.info("The sim speed is already limited to 256kHz")
            return False

        logger.info("Limiting the sim speed to 256 kHz")
        self.send_command(f'AT+UDCONF=86,7,{SIM_SPEED_256K}'.encode('ascii'), no_answer=True)
        return True

    def get_mode(self):
        """ Get function mode (router or bridge) """
        out = self.send_command(b'AT+UBMCONF?')

        x = re.search('UBMCONF:\\s(\\d)', out)
        mode = int(x.group(1))
        if mode == 1:
            return "Router"
        elif mode == 2:
            return "Bridge"
        else:
            return "unsupported"

    def set_mode(self, mode):
        """ Returns True if a reset is required """
        actual = self.get_mode()
        if actual != mode:
            if mode == 'Bridge':
                self.send_command(b'AT+UBMCONF=2', no_answer=True)
            elif mode == 'Router':
                self.send_command(b'AT+UBMCONF=1', no_answer=True)
            else:
                raise Exception('mode not supported')
            return True
        return False

    def get_usb_profile(self):
        """ Get USB profile (ECM or RNDIS) """
        out = self.send_command(b'AT+UUSBCONF?')
        x = re.search('UUSBCONF:\\s\\d,"(.*)",', out)
        return x.group(1)

    def set_usb_profile(self, profile):
        """ Returns True if a reset is required """
        actual = self.get_usb_profile()
        if actual != profile:
            if profile == 'ECM':
                self.send_command(b'AT+UUSBCONF=2,"ECM",0', no_answer=True)
            elif profile == 'RNDIS':
                self.send_command(b'AT+UUSBCONF=3', no_answer=True)
            else:
                raise Exception('mode not supported')
            return True
        return False

    @staticmethod
    def _firmware_strtoi(fw_str):
        """ Convert firmware string to comparable integer """
        if ',A' in fw_str:  # Separation between firmware and app versions
            fw_str = fw_str.replace(',A', '')
        else:
            fw_str += '0000'  # Default App version
        try:
            return int(fw_str.replace('.', ''))
        except ValueError:
            raise UnexpectedFirmwareVersion()

    def _get_firmware_number(self):
        """ Returns the firmware as comparable int """
        fw = super().get_firmware()
        return self._firmware_strtoi(fw)

    def get_firmware(self):
        """ Returns the fw as string and an indication of its "quality" """
        fw = super().get_firmware()

        try:
            fw_nbr = self._firmware_strtoi(fw)
        except UnexpectedFirmwareVersion:
            return fw, WwanModem.UNTESTED_FW

        rec_fw_nbr = self._firmware_strtoi(RECOMMENDED_FW)
        last_fw_nbr = self._firmware_strtoi(LATEST_FW)

        if fw_nbr > last_fw_nbr:
            return fw, WwanModem.UNTESTED_FW
        elif fw_nbr == last_fw_nbr:
            return fw, WwanModem.LATEST_FW
        elif fw_nbr >= rec_fw_nbr:
            return fw, WwanModem.DEP_FW
        else:
            return fw, WwanModem.OBS_FW
