#!/usr/bin/env python3

import serial
import os
import time
import re

from .exceptions import *
from .logger import logger
# Feature not available on all platforms
try:
    from .platform import modem_voltage
except ImportError:
    modem_voltage = None
try:
    from .platform import PWR_ALWAYS_ON
except ImportError:
    PWR_ALWAYS_ON = False


POWERON_MAX_DELAY_S = 15
POWEROFF_MAX_DELAY_S = 7.5

class WwanModem:
    LATEST_FW = 0
    DEP_FW = 1  # Deprecated: still supported but upgrade possible
    OBS_FW = 2  # Obsolete: not supported anymore
    UNTESTED_FW = 3

    serial = None
    name = None
    pwr = None

    def __init__(self, device_name):
        if isinstance(device_name, WwanModem):
            self.__dict__.update(device_name.__dict__)
            return

        self.name = device_name

        while True:
            try:
                self.power_up()
                break
            except ModemNotPoweredException as e:
                logger.error("The modem did not power up, turning power off and on again")
                self.power_down()
                # Make sure the power is down long enough to avoid a glitch
                # With toby-l2 this down time must be at least 130ms
                time.sleep(1)

        self.open_serial()

    def get_name(self):
        return self.name

    def get_serial_name(self):
        return '/dev/{}'.format(self.name)

    def send_command(self, cmd, no_answer=False):
        retry = False
        while True:
            self.serial.flushInput()
            self.serial.write(cmd)
            self.serial.write(b'\r')
            s = self.serial.read_until(b'OK')
            if(len(s.decode('utf-8')) < 4):
                # Avoid empty response just after reset
                continue
            if b'ERROR' in s:
                raise AtCommandReturnedError()
            elif not b'OK' in s and retry:
                raise AtCommandException("Unhandled answer from modem : {}".format(s))
            elif not b'OK' in s and not retry:
                logger.warn("Modem did not answer properly, retrying command")
                # Did not receive a proper response
                retry = True
                continue
            else:
                # Proper answer
                break

        if no_answer:
            return

        x = re.search('(.*)\n.*\nOK', s.decode('utf-8').replace('\r', ''))
        if not x:
            raise AtCommandException("Modem answer not parsed properly")
        return x.group(1)

    def power_down(self):
        logger.info("modem power down")

        if PWR_ALWAYS_ON:
            logger.info("The power cannot be controlled, not shutting down");
            return

        if not os.path.exists(self.get_serial_name()):
            logger.warn("The modem is already off. Making sure that power is cut")
            self.set_pwr_state(0)
            return

        self.serial.write(b'AT+CPWROFF\r')

        timeout_s = POWEROFF_MAX_DELAY_S
        if modem_voltage is not None and self.name in modem_voltage:
            sleep_time = 0.5
            while open(modem_voltage[self.name], 'r').read()[:-1] != '0':
                time.sleep(sleep_time)
                timeout_s -= sleep_time
                if timeout_s <= 0:
                    logger.error("Timeout: modem did not turn off by itself")
                    break
        else:
            logger.info("Using default poweroff delay")
            time.sleep(POWEROFF_MAX_DELAY_S)

        self.set_pwr_state(0)
        self.wait_on_down()

    def get_vendor(self):
        return self.send_command(b'AT+CGMI')

    def get_model(self):
        return self.send_command(b'AT+CGMM')

    def get_firmware(self):
        return self.send_command(b'ATI9')

    def get_sim_id(self):
        try:
            return self.send_command(b'AT+CCID?')
        except AtCommandReturnedError:
            raise NoSimException()


    def close_serial(self):
        self.serial.close()

    def open_serial(self):
        serial_name = self.get_serial_name()
        if os.path.exists(serial_name):
            self.serial = serial.Serial(serial_name, 115200, timeout=2)
        else:
            raise Exception("Serial device does not exist")

    def wait_on_down(self):
        while os.path.exists(self.get_serial_name()):
            time.sleep(1)

    def wait_on_up(self):
        timeout_s = POWERON_MAX_DELAY_S;
        while not os.path.exists(self.get_serial_name()):
            time.sleep(1)
            timeout_s -= 1
            if timeout_s == 0:
                raise ModemNotPoweredException()

    def set_pwr_state(self, value):
        state = "enabled" if value == 1 else "disabled"
        with open("/sys/devices/platform/" + self.name + "-supply/state", "w") as supply_file:
            supply_file.write(state + "\n")

    def power_up(self):
        logger.info("modem power up")
        if os.path.exists(self.get_serial_name()):
            # Modem already powered
            return

        if not PWR_ALWAYS_ON:
            self.set_pwr_state(1)

        self.wait_on_up()




def main():
    modem = WwanModem('wwan0')
    print(modem.get_vendor())
    print(modem.get_model())
    print(modem.get_firmware())
    print(modem.get_sim_id())


if __name__ == '__main__':
    main()
